/*
  STM32_disco_ball

  Disco Ball

  uses the "small" STM32F103

  for use with STM32F103
    -- STM32F103 (blue pill)
    ---------------------
    File --> preferences --> additional URL:https://github.com/stm32duino/BoardManagerFiles/raw/main/package_stmicroelectronics_index.json
    Tools --> BoardManager --> search for "STM32" --> install

    Tools --> Boards --> STM32 boards group --> Generic STM32F1 series
    Tools --> Boardpartnumber --> BluepillF103C8
    Tools --> Upload method --> STM32CubeProgrammerSWD (STM32IDE already installed)

    The "original" bluepill need the jumpers to be set for Programming (the "small" one NO!)
    The "original" bluepill has the LED placed at PC13, the "small" has it in PB12


*/


// Addressable LED specific lines
//-------------------------------

#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
#include <avr/power.h> // Required for 16 MHz Adafruit Trinket
#endif

// Which pin on the Arduino is connected to the NeoPixels?
#define PIN        PB7   //PB7 for STM103,  2 for Arduino Nano

// How many NeoPixels are attached to the Arduino?
#define NUMPIXELS 400// Popular NeoPixel ring size

// When setting up the NeoPixel library, we tell it how many pixels,
// and which pin to use to send signals. Note that for older NeoPixel
// strips you might need to change the third parameter -- see the
// strandtest example for more information on possible values.
Adafruit_NeoPixel pixels(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);

//int DELAYVAL=500; // Time (in milliseconds) to pause between pixels

// end-of-addressable LED specific lines


// array for colors
uint32_t colorset[20];

uint32_t disco[32][13];     // The complete setup of the disco-ball
// 32 lines of max 13 leds/line
// even lines have 13 active leds,
// odd lines have 12 active leds
// enumeration starts from bottom of the ball

void clear_disco(void)
{
  int i, j;

  for (i = 0; i < 32; i++)
    for (j = 0; j < 13; j++)
      disco[i][j] = 0;
}
//-------------------------------------------------------------------
void show_disco(void)
{
  int i, j;


  pixels.clear(); // Set all pixel colors to 'off'

  for (i = 0; i < 32; i++)
  {
    if (i % 2 == 0)
    {
      for (j = 0; j < 12; j++)
        pixels.setPixelColor((i / 2) * 25 + (11 - j), disco[i][j]);
    }
    else
    {
      for (j = 0; j < 13; j++)
        pixels.setPixelColor((i / 2) * 25 + (12 + j), disco[i][j]);
    }
  }

  pixels.show();

}

//-------------------------------------------------------------------
void show_disco_shifted(int shift)
{
  int i, j;


  pixels.clear(); // Set all pixel colors to 'off'

  for (i = 0; i < 32; i++)
  {
    if (i % 2 == 0)
    {
      for (j = 0; j < 12; j++)
        pixels.setPixelColor((i / 2) * 25 + (11 - j), disco[(i+shift)%32][j]);
    }
    else
    {
      for (j = 0; j < 13; j++)
        pixels.setPixelColor((i / 2) * 25 + (12 + j), disco[(i+shift)%32][j]);
    }
  }

  pixels.show();

}

//-------------------------------------------------------------------
// the setup function runs once when you press reset or power the board
void setup() {
  int i;

  // flash LED initialization
  //--------------------------
  pixels.begin(); // INITIALIZE NeoPixel strip object (REQUIRED)


  colorset[0] = pixels.Color(250, 0, 0); // red
  colorset[1] = pixels.Color(0, 250, 0); // green
  colorset[2] = pixels.Color(0, 0, 250); // blue
  colorset[3] = pixels.Color(200, 200, 0); // yellow
  colorset[4] = pixels.Color(100, 100, 100); // white
  colorset[5] = pixels.Color(140, 40, 40); // brown
  colorset[6] = pixels.Color(0, 0, 0); // black
  colorset[7] = pixels.Color(250, 0, 0); // red
  colorset[8] = pixels.Color(200, 200, 0); // yellow
  colorset[9] = pixels.Color(0, 0, 250); // blue
  colorset[10] = pixels.Color(50, 0, 0); // red
  colorset[11] = pixels.Color(0, 50, 0); // green
  colorset[12] = pixels.Color(0, 0, 50); // blue
  colorset[13] = pixels.Color(40, 40, 0); // yellow
  colorset[14] = pixels.Color(20, 20, 20); // white
  colorset[15] = pixels.Color(30, 8, 8); // brown


  clear_disco();
  show_disco();


}


//-------------------------------------------------------------------
void h_zones(void)
{
  for (int j = 0; j < 12; j++)
  {
    for (int i = 0; i < 32; i++)
    {
      disco[i][j] = colorset[j % 6];
    }

    show_disco();
    delay(70);


    for (int i = 0; i < 32; i++)
    {
      disco[i][j] = 0;
    }

  }
}
//-------------------------------------------------------------------
void sides(void)
{

  for (int i = 0; i < 32; i = i + 2)
  {
    for (int j = 0; j < 12; j++)
    {
      disco[i][j] = colorset[10 + (i / 2 + j) % 4]; //low brightness colors
      disco[i + 1][j] = colorset[10 + (i / 2 + j) % 4]; //low brightness colors
    }
  }
  // show_disco();
  //  delay(10000);

  for (;;)
  {
    for (int k = 0 ; k < 50; k++)
    {
      show_disco();
      delay(300);

      for ( int g = 0; g < 2; g++)
      {
        // shift right
        uint32_t temp;
        for (int j = 0; j < 12; j++)
        {
          temp = disco[31][j];
          for (int i = 31; i > 0; i--)
          {
            disco[i][j] = disco[i - 1][j];
          }
          disco[0][j] = temp;
        }
      }// for g
    } // for k
  }



}
//-------------------------------------------------------------------
void shine(void)
{
  uint32_t colors[20];

  // power measurements
  // 50,50,50 = 1.4A
  // 100,100,100 = 1.62 A (color shift..)
  // 150,150,150 = 1.77 A (color shift. Voltage?)
  // 200,200,200 = 1.87 A

  colors[0] = pixels.Color(100, 100, 100); // low brightness white
  colors[1] = pixels.Color(250, 50, 50); // low brightness white + red
  colors[2] = pixels.Color(50, 250, 50); // low brightness white + green
  colors[3] = pixels.Color(50, 50, 250); // low brightness white + blue
  colors[4] = pixels.Color(250, 250, 50); // low brightness white + yellow
  colors[5] = pixels.Color(250, 50, 250); // low brightness white + purple
  colors[6] = pixels.Color(250, 100, 100); // low brightness white + brown



  // set background at a low brightness white
  for (int i = 0; i < 32; i++)
  {
    for (int j = 0; j < 12; j++)
    {
      disco[i][j] = colors[0]; //low brightness white
    }
  }


  for (int k = 0; k < 50; k++)
  {
    disco[rand() % 31][rand() % 12] = colors[rand() % 6 + 1];
  }


  show_disco();
  delay(200);


}
//-------------------------------------------------------------------
void set_h_line(int line, uint32_t colorX)
/*
 *  sets horizontal line <line> to color <colorX>
 */
{
  
    for (int i = (line + 1) & 1; i < 32; i = i + 2)
    {
      disco[i][line >> 1] = colorX;
    }

}
//-------------------------------------------------------------------
void h_line_up(uint32_t colorB, uint32_t colorX, int D)
/*
 * <colorX> is the color of the horizontal line
 * <colorB> is the color of the background
 * <D> is the delay in ms
 */
{
  uint32_t colors[20];

  colors[0] = pixels.Color(100, 100, 100); // low brightness white
  colors[1] = pixels.Color(250, 0, 0); // low brightness white + red
  colors[2] = pixels.Color(50, 250, 50); // low brightness white + green
  colors[3] = pixels.Color(50, 50, 250); // low brightness white + blue
  colors[4] = pixels.Color(250, 250, 50); // low brightness white + yellow
  colors[5] = pixels.Color(250, 50, 250); // low brightness white + purple
  colors[6] = pixels.Color(250, 100, 100); // low brightness white + brown


  for (int k = 0  ; k < 25; k++)
  {

    // set all leds to background white
    for (int j = 0; j < 13; j++)
    {
      for (int i = 0; i < 32; i++)
      {
        disco[i][j] = colorB;
      }
    }

    // set the <k> line to a color

    set_h_line(k,colorX);

    
    show_disco();
    delay(D);

  } // next <k>


}
//-------------------------------------------------------------------
void h_line_down(uint32_t colorB, uint32_t colorX, int D)
/*
 * <colorX> is the color of the horizontal line
 * <colorB> is the color of the background
 * <D> is the delay in ms
 */
{
  uint32_t colors[20];

  colors[0] = pixels.Color(100, 100, 100); // low brightness white
  colors[1] = pixels.Color(250, 0, 0); // low brightness white + red
  colors[2] = pixels.Color(50, 250, 50); // low brightness white + green
  colors[3] = pixels.Color(50, 50, 250); // low brightness white + blue
  colors[4] = pixels.Color(250, 250, 50); // low brightness white + yellow
  colors[5] = pixels.Color(250, 50, 250); // low brightness white + purple
  colors[6] = pixels.Color(250, 100, 100); // low brightness white + brown


  for (int k = 0  ; k < 25; k++)
  {

    // set all leds to background white
    for (int j = 0; j < 13; j++)
    {
      for (int i = 0; i < 32; i++)
      {
        disco[i][j] = colorB;
      }
    }

    // set the <k> line to a color

    for (int i = (k) % 2; i < 32; i = i + 2)
    {
      disco[i][12 - k / 2] = colorX;
    }

    show_disco();
    delay(D);

  } // next <k>


}
//-------------------------------------------------------------------
void h_lines_down(uint32_t colorB, uint32_t colorX, int D)
/*
 * multiple horizontal lines running from top to bottom
 * 
 * there are 25 horizontal lines (12+13)
 * 
 * <colorX> is the color of the horizontal line
 * <colorB> is the color of the background
 * <D> is the delay in ms
 */
{
  uint32_t colors[20];

  colors[0] = pixels.Color(100, 100, 100); // low brightness white
  colors[1] = pixels.Color(250, 0, 0); // low brightness white + red
  colors[2] = pixels.Color(50, 250, 50); // low brightness white + green
  colors[3] = pixels.Color(50, 50, 250); // low brightness white + blue
  colors[4] = pixels.Color(250, 250, 50); // low brightness white + yellow
  colors[5] = pixels.Color(250, 50, 250); // low brightness white + purple
  colors[6] = pixels.Color(250, 100, 100); // low brightness white + brown



  int k=24;     //start point at top
  int k1,k2;
  for (int count = 0  ; count < 200; count++)      //25 line steps X 8 cycles
  {
  // set all leds to background color <colorB>
  for (int j = 0; j < 13; j++)
    for (int i = 0; i < 32; i++)
      disco[i][j] = colorB;


    // set the <k> line to a color
    set_h_line(k,colorX);
    

    // set the <k1> line if possible
    k1=k+10;
    if (k1<=24)
       set_h_line(k1,colorX);
    
    // set the <k2> line if possible
    k2=k+20;
    if (k2<=24)
      set_h_line(k2,colorX);
      

    show_disco();
    delay(D);


    // next k
    k--;      // going down
    if (k<0) k=k1-1;
     
  } // next  step


}
//-------------------------------------------------------------------
void h_lines_up_down(uint32_t colorB, int D,int mode)
/*
 * multiple horizontal lines running from top to bottom and vice versa
 * 
 * there are 25 horizontal lines (12+13)
 * 
 * <colorB> is the color of the background
 * <D> is the delay in ms
 * if <mode>==1 then 1UP-1DOWN, else 4fastUP-4fastDOWN
 */
{
  uint32_t colors[20];
  colors[0] = pixels.Color(250, 0, 0); // red
  colors[1] = pixels.Color(0, 250, 0); // green
  colors[2] = pixels.Color(0, 0, 250); // blue
  colors[3] = pixels.Color(200, 200, 0); // yellow
  colors[4] = pixels.Color(200, 0, 200); // purple
  colors[5] = pixels.Color(0, 200, 200); // light blue

 uint32_t linecolor[9];    // color of each line

 for (int i=0; i<9; i++)
   linecolor[i]=colors[rand()%6];

 int N,r;

 if (mode==1) {N=100; r=1;} else {N=15; r=6;}

 for (int z=0;z<r;z++)
 {

  // UP runing
  int k=0;     //start point at bottom
  for (int count = 0  ; count < N; count++)      //25 line steps X 6 cycles
  {
  // set all leds to background color <colorB>
  for (int j = 0; j < 13; j++)
    for (int i = 0; i < 32; i++)
      disco[i][j] = colorB;


    // set every 3*<k> line to a color
    for (int i=0; i<9; i++)
      if (3*i+k < 25)
        set_h_line(3*i+k,linecolor[i]);
    

    show_disco();
    delay(D);

    // next k
    k++;      // going up
    if (k>2) 
    {
      k=0;    // start over
      // shift linecolor and add a new color
      for (int i = 8; i>0 ; i--)
        linecolor[i]=linecolor[i-1];
      linecolor[0]=colors[rand()%6];
    }
     
  } // next  step


  // DOWN runing
  k=2;     //start point at top
  for (int count = 0  ; count < N; count++)      //25 line steps X 6 cycles
  {
  // set all leds to background color <colorB>
  for (int j = 0; j < 13; j++)
    for (int i = 0; i < 32; i++)
      disco[i][j] = colorB;

    // set every 3*<k> line to a color
    for (int i=0; i<9; i++)
      if (3*i+k < 25)
        set_h_line(3*i+k,linecolor[i]);
    


    show_disco();
    delay(D);


    // next k
    k--;      // going down
    if (k<0) 
    {
      k=2;    // start over
      // shift linecolor and add a new color
      for (int i = 0; i<8 ; i++)
        linecolor[i]=linecolor[i+1];
      linecolor[8]=colors[rand()%6];
    }
     
  } // next  step

 }// next z
}
//-------------------------------------------------------------------
void slash(uint32_t color, int Start)
/*
 *  setup a slash, starting at the bottom at position <start> (0 to 31)
 *  going UP-RIGHT
 *  ONLY even <Start> values are allowe
 *  use <color>
 * 
 */
{
      for (int j = 0; j < 13; j++)
      {
          disco[(Start+2*j)%32][j] = color;
          disco[(Start+2*j+31)%32][j] = color;
      }
  
}
//-------------------------------------------------------------------
void slash_LR(uint32_t colorB, int D,int mode)
/*
 * multiple slashes running from Left to Right and vice versa
 * 
 * there are 25 horizontal lines (12+13)
 * 
 * <colorB> is the color of the background
 * <D> is the delay in ms
 * if <mode>==1 then 1R-1L, else 4fastR-4fastL
 */
{
  uint32_t colors[20];
  colors[0] = pixels.Color(250, 0, 0); // red
  colors[1] = pixels.Color(0, 250, 0); // green
  colors[2] = pixels.Color(0, 0, 250); // blue
  colors[3] = pixels.Color(200, 200, 0); // yellow
  colors[4] = pixels.Color(200, 0, 200); // purple
  colors[5] = pixels.Color(0, 200, 200); // light blue

 uint32_t linecolor[16];    // color of each line
 uint32_t temp;
 
 for (int i=0; i<16; i++)
   linecolor[i]=colors[rand()%6];

 int N,r;

 if (mode==1) {N=30; r=1;} else {N=10; r=5;}

 for (int z=0;z<r;z++)
 {

  // UP runing
  for (int count = 0  ; count < N; count++)      //25 line steps X 6 cycles
  {
  // set all leds to background color <colorB>
  for (int j = 0; j < 13; j++)
    for (int i = 0; i < 32; i++)
      disco[i][j] = colorB;

/*
    // set every even slash line
    for (int i=0; i<16; i++)
        slash(linecolor[i],i*2);
*/     
    for (int i=0; i<8; i++)
        slash(linecolor[i],i*4);

    show_disco();
    delay(D);

      // rotate linecolor
      temp=linecolor[15];
      for (int i = 15; i>0 ; i--)
        linecolor[i]=linecolor[i-1];
      linecolor[0]=temp;
     
  } // next  step


  // Left runing
  for (int count = 0  ; count < N; count++)      //25 line steps X 6 cycles
  {
  // set all leds to background color <colorB>
  for (int j = 0; j < 13; j++)
    for (int i = 0; i < 32; i++)
      disco[i][j] = colorB;

/*
    // set every even slash line
    for (int i=0; i<16; i++)
        slash(linecolor[i],i*2);
*/     
    for (int i=0; i<8; i++)
        slash(linecolor[i],i*4);

    show_disco();
    delay(D);


      // rotate linecolor
      temp=linecolor[0];
      for (int i = 0; i<15 ; i++)
        linecolor[i]=linecolor[i+1];
      linecolor[15]=temp;
     
  } // next  step

 }// next z
}
//-------------------------------------------------------------------
void slash_LR_fat(uint32_t colorB, int D,int mode)
/*
 * multiple slashes running from Left to Right and vice versa
 * 
 * there are 25 horizontal lines (12+13)
 * 
 * <colorB> is the color of the background
 * <D> is the delay in ms
 * if <mode>==1 then 1R-1L, else 4fastR-4fastL
 */
{
  uint32_t colors[20];
  colors[0] = pixels.Color(250, 0, 0); // red
  colors[1] = pixels.Color(0, 250, 0); // green
  colors[2] = pixels.Color(0, 0, 250); // blue
  colors[3] = pixels.Color(200, 200, 0); // yellow
  colors[4] = pixels.Color(200, 0, 200); // purple
  colors[5] = pixels.Color(0, 200, 200); // light blue

 uint32_t linecolor[16];    // color of each line
 uint32_t temp;
 
 for (int i=0; i<4; i++)
   linecolor[i]=colors[rand()%6];

 int N,r;

 if (mode==1) {N=10; r=1;} else {N=3; r=5;}

 for (int z=0;z<r;z++)
 {

  // Right runing
  for (int count = 0  ; count < N; count++)      //25 line steps X 6 cycles
  {

    for (int k=0;k<8;k=k+2)
    {
  // set all leds to background color <colorB>
  for (int j = 0; j < 13; j++)
    for (int i = 0; i < 32; i++)
      disco[i][j] = colorB;


    for (int i=0; i<4; i++)
    {
        slash(linecolor[i],i*8+k);
        slash(linecolor[i],i*8+2+k);
    }
    
    show_disco();
    delay(D);
    }
    
      // shift linecolor
      for (int i = 3; i>0 ; i--)
        linecolor[i]=linecolor[i-1];
      linecolor[0]=colors[rand()%6];
     
  } // next  step


  // Left runing
  for (int count = 0  ; count < N; count++)      //25 line steps X 6 cycles
  {

    for (int k=6;k>=0;k=k-2)
    {
  // set all leds to background color <colorB>
  for (int j = 0; j < 13; j++)
    for (int i = 0; i < 32; i++)
      disco[i][j] = colorB;


    for (int i=0; i<4; i++)
    {
        slash(linecolor[i],i*8+k);
        slash(linecolor[i],i*8+2+k);
    }
    
    show_disco();
    delay(D);
    }


      // shift linecolor
      for (int i = 0; i<3 ; i++)
        linecolor[i]=linecolor[i+1];
      linecolor[3]=colors[rand()%6];
     
  } // next  step

 }// next z
}
//-------------------------------------------------------------------
void vertical(uint32_t colorB, int D,int mode)
/*
 * multiple vertical running from Left to Right and vice versa
 * 
 * 
 * <colorB> is the color of the background
 * <D> is the delay in ms
 * if <mode>==1 then 1R-1L, else 4fastR-4fastL
 */
{
  uint32_t colors[20];
  colors[0] = pixels.Color(250, 0, 0); // red
  colors[1] = pixels.Color(0, 250, 0); // green
  colors[2] = pixels.Color(0, 0, 250); // blue
  colors[3] = pixels.Color(200, 200, 0); // yellow
  colors[4] = pixels.Color(200, 0, 200); // purple
  colors[5] = pixels.Color(0, 200, 200); // light blue

 uint32_t linecolor[16];    // color of each line
 uint32_t temp;
 
 for (int i=0; i<16; i++)
   linecolor[i]=colors[rand()%6];

 int N,r;

 if (mode==1) {N=30; r=1;} else {N=10; r=5;}

 for (int z=0;z<r;z++)
 {

  // RIGHT runing
  for (int count = 0  ; count < N; count++)      //25 line steps X 6 cycles
  {
  // set all leds to background color <colorB>
  for (int j = 0; j < 13; j++)
    for (int i = 0; i < 32; i++)
      disco[i][j] = colorB;


    // set every even vertical line
    for (int i=0; i<16; i++)
      for (int j=0;j<13;j++)
          disco[i*2][j] = linecolor[i];
     
    show_disco();
    delay(D);

  // set all leds to background color <colorB>
  for (int j = 0; j < 13; j++)
    for (int i = 0; i < 32; i++)
      disco[i][j] = colorB;

    
    // set every odd vertical line
    for (int i=0; i<16; i++)
      for (int j=0;j<13;j++)
          disco[i*2+1][j] = linecolor[i];
     
    show_disco();
    delay(D);

      // rotate linecolor
      temp=linecolor[15];
      for (int i = 15; i>0 ; i--)
        linecolor[i]=linecolor[i-1];
      linecolor[0]=temp;
     
  } // next  step


  // Left runing
  for (int count = 0  ; count < N; count++)      //25 line steps X 6 cycles
  {
  // set all leds to background color <colorB>
  for (int j = 0; j < 13; j++)
    for (int i = 0; i < 32; i++)
      disco[i][j] = colorB;

    // set every even vertical line
    for (int i=0; i<16; i++)
      for (int j=0;j<13;j++)
          disco[i*2][j] = linecolor[i];

    show_disco();
    delay(D);

  // set all leds to background color <colorB>
  for (int j = 0; j < 13; j++)
    for (int i = 0; i < 32; i++)
      disco[i][j] = colorB;

    // set every odd vertical line
    for (int i=0; i<16; i++)
      for (int j=0;j<13;j++)
          disco[i*2+1][j] = linecolor[i];

    show_disco();
    delay(D);


      // rotate linecolor
      temp=linecolor[0];
      for (int i = 0; i<15 ; i++)
        linecolor[i]=linecolor[i+1];
      linecolor[15]=temp;
     
  } // next  step

 }// next z
}
//-------------------------------------------------------------------
void flash(uint32_t colorB, int N, int D)
/*
 * <colorX> is the color 
 * <N> is the number of flashes
 * <D> is the delay in ms
 */
{

  for (int k = 0  ; k < N; k++)
  {

    // set all leds ON
    for (int j = 0; j < 13; j++)
    {
      for (int i = 0; i < 32; i++)
      {
        disco[i][j] = colorB;
      }
    }

    show_disco();
    delay(D*3);


    // set all leds OFF
    for (int j = 0; j < 13; j++)
    {
      for (int i = 0; i < 32; i++)
      {
        disco[i][j] = pixels.Color(0,0,0);
      }
    }

    show_disco();
    delay(D*3);

  } // next <k>


}
//-------------------------------------------------------------------
void flash_set(int d)
{
  flash(pixels.Color(250,0,0),2,d);
  flash(pixels.Color(200,200,200),2,d);
  flash(pixels.Color(0,250,0),2,d);
  flash(pixels.Color(200,200,200),2,d);
  flash(pixels.Color(0,0,250),2,d);
  flash(pixels.Color(250,0,0),1,d);
  flash(pixels.Color(0,250,0),1,d);
  flash(pixels.Color(0,0,250),1,d);
  flash(pixels.Color(250,0,0),1,d);
  flash(pixels.Color(0,250,0),1,d);
  flash(pixels.Color(0,0,250),1,d);
  
  
}
//-------------------------------------------------------------------
void stars(int D)
{
#define N 80
#define M 8
#define L 10
// M*L=N


  uint32_t colors[20];
  colors[0] = pixels.Color(250, 0, 0); // red
  colors[1] = pixels.Color(0, 250, 0); // green
  colors[2] = pixels.Color(0, 0, 250); // blue
  colors[3] = pixels.Color(200, 200, 0); // yellow
  colors[4] = pixels.Color(200, 0, 200); // purple
  colors[5] = pixels.Color(0, 200, 200); // light blue

 uint32_t starcolor[N];    // stars
 int starX[N], starY[N];
 
 uint32_t colorB=pixels.Color(150,150,150);
 
 
  // set all leds to background color <colorB>
  for (int j = 0; j < 13; j++)
    for (int i = 0; i < 32; i++)
      disco[i][j] = colorB;

 for (int i=0; i<N; i++)
 {
   starcolor[i]=colors[rand()%6];
   starX[i]=rand()%32;
   starY[i]=rand()%13;

   disco[starX[i]][starY[i]]=starcolor[i];
 }


 int i=0;
  for (int count = 0  ; count < 150; count++)     
  {
    show_disco();
    delay(D);
  for (int k=0; k<M; k++)
  {
   disco[starX[i*M+k]][starY[i*M+k]]=colorB;

   starcolor[i*M+k]=colors[rand()%6];
   starX[i*M+k]=rand()%32;
   starY[i*M+k]=rand()%13;

   disco[starX[i*M+k]][starY[i*M+k]]=starcolor[i*M+k];
  }
    i++;
    if (i==L)i=0;
  }  
}
//-------------------------------------------------------------------
void stars_rotate(int D)
{
#define N 80
#define M 8
#define L 10
// M*L=N


  uint32_t colors[20];
  colors[0] = pixels.Color(250, 0, 0); // red
  colors[1] = pixels.Color(0, 250, 0); // green
  colors[2] = pixels.Color(0, 0, 250); // blue
  colors[3] = pixels.Color(200, 200, 0); // yellow
  colors[4] = pixels.Color(200, 0, 200); // purple
  colors[5] = pixels.Color(0, 200, 200); // light blue

 uint32_t starcolor[N];    // stars
 int starX[N], starY[N];
 
 uint32_t colorB=pixels.Color(150,150,150);
 
 
  // set all leds to background color <colorB>
  for (int j = 0; j < 13; j++)
    for (int i = 0; i < 32; i++)
      disco[i][j] = colorB;

 for (int i=0; i<N; i++)
 {
   starcolor[i]=colors[rand()%6];
   starX[i]=rand()%32;
   starY[i]=rand()%13;

   disco[starX[i]][starY[i]]=starcolor[i];
 }

int sc=0;

 int i=0;
  for (int count = 0  ; count < 150; count++)     
  {
    show_disco_shifted(sc/2);  sc++; if(sc==64)sc=0;
    delay(D);
  for (int k=0; k<M; k++)
  {
   disco[starX[i*M+k]][starY[i*M+k]]=colorB;

   starcolor[i*M+k]=colors[rand()%6];
   starX[i*M+k]=rand()%32;
   starY[i*M+k]=rand()%13;

   disco[starX[i*M+k]][starY[i*M+k]]=starcolor[i*M+k];
  }
    i++;
    if (i==L)i=0;
  }  
}
//-------------------------------------------------------------------
// the loop function runs over and over again forever
void loop() {
int d;
 // d= analogRead(A0);
 d=80; 
  //h_zones();
  //sides();
  //shine();

  // multi-horizontal UP-DOWN set
/*   h_line_up(pixels.Color(50,50,50), pixels.Color(250,0,0),d);
   h_line_up(pixels.Color(50,50,50), pixels.Color(0,250,0),d);
*/

/*
   h_line_up(pixels.Color(0,50,50), pixels.Color(250,0,0),d);
   h_line_up(pixels.Color(50,0,50), pixels.Color(0,250,0),d);
*/


  stars(d);
 
  h_lines_up_down(pixels.Color(0,0,0),d,1);

  slash_LR_fat(pixels.Color(0,10,0),d,2);
  
  vertical(pixels.Color(0,0,3),d,1);
  
  flash_set(d);

  slash_LR_fat(pixels.Color(0,0,10),d,1);

  vertical(pixels.Color(0,0,0),d,2);

  //slash_LR_fat(pixels.Color(0,10,0),d,2);

  h_lines_up_down(pixels.Color(0,0,0),d,2);

  //h_lines_down(pixels.Color(0,0,0), pixels.Color(0,250,0),d);
  //h_line_down(pixels.Color(0,0,0), pixels.Color(250,0,0),d);
  //h_line_down(pixels.Color(0,0,0), pixels.Color(0,250,0),d);

}
